<?php
/**
 * Plugin Name: FleekCode - Omnibus Price Tracker
 * Plugin URI: https://wordpress.org/plugins/fleekcode-omnibus/
 * Description: Automatically tracks and displays the minimum (reference) price over a specified number of days in compliance with Omnibus requirements.
 * Version: 1.0.3
 * Author: Fleekcode
 * Author URI: https://profiles.wordpress.org/fleekcode/
 * License: GPLv2 or later
 * Text Domain: fleekcode-omnibus
 * Domain Path: /languages
 */

defined('ABSPATH') || exit;

define('FLEEKCODE_OMNIBUS_VERSION', get_file_data(__FILE__, ['Version' => 'Version'])['Version']);
define('FLEEKCODE_OMNIBUS_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('FLEEKCODE_OMNIBUS_PLUGIN_URL', plugin_dir_url(__FILE__));

require_once FLEEKCODE_OMNIBUS_PLUGIN_DIR . 'includes/class-activator.php';
require_once FLEEKCODE_OMNIBUS_PLUGIN_DIR . 'includes/class-database.php';
require_once FLEEKCODE_OMNIBUS_PLUGIN_DIR . 'includes/class-core.php';
require_once FLEEKCODE_OMNIBUS_PLUGIN_DIR . 'admin/class-admin.php';
require_once FLEEKCODE_OMNIBUS_PLUGIN_DIR . 'public/class-public.php';

register_activation_hook(__FILE__, array('Fleekcode_Activator', 'activate'));
register_deactivation_hook(__FILE__, 'fleekcode_omnibus_deactivate');
register_uninstall_hook(__FILE__, 'fleekcode_omnibus_uninstall');

add_action('plugins_loaded', 'fleekcode_omnibus_init');

/**
 * Initialize the plugin
 */
function fleekcode_omnibus_init() {
    load_plugin_textdomain(
        'fleekcode-omnibus',
        false,
        dirname(plugin_basename(__FILE__)) . '/languages'
    );

    if (!class_exists('WooCommerce')) {
        add_action('admin_notices', 'fleekcode_omnibus_missing_wc_notice');
        return;
    }

    if (version_compare(WC()->version, '3.0', '<')) {
        add_action('admin_notices', 'fleekcode_omnibus_wc_version_notice');
        return;
    }

    Fleekcode_Database::init();
    Fleekcode_Core::init();
    Fleekcode_Admin::init();
    Fleekcode_Public::init();
}

/**
 * Shortcode to display the minimum price tracked by Omnibus
 *
 * @param array $atts Shortcode attributes
 * @return string HTML output with the minimum price
 */
function fleekcode_omnibus_price_shortcode($atts) {
    $atts = shortcode_atts(array(
        'product_id' => null,
    ), $atts, 'fleekcode_omnibus_price');

    if ($atts['product_id']) {
        $product = wc_get_product(intval($atts['product_id']));
    } else {
        global $post;
        $product = wc_get_product($post->ID);
    }

    if (!$product || !is_a($product, 'WC_Product')) {
        return '';
    }

    $display_method = get_option('fleekcode_omnibus_display_method', 'default');
    $display_location = get_option('fleekcode_omnibus_display_location', 'both');
    $days = absint(get_option('fleekcode_omnibus_days', 30));
    $min_price_display_mode = get_option('fleekcode_omnibus_min_price_display_mode', 'always');
    $text_template = get_option('fleekcode_omnibus_text', __('Minimum price over %d days: %price%', 'fleekcode-omnibus'));

    if (('product' === $display_location && !is_product()) ||
        ('catalog' === $display_location && is_product())) {
        return ''; 
    }

    if ('sale' === $min_price_display_mode && !$product->is_on_sale()) {
        return ''; 
    }

    $reference_price = Fleekcode_Database::get_reference_price($product);
    if (false === $reference_price || $reference_price <= 0) {
        return '';
    }

    $custom_text = str_replace(
        array('%d', '%price%'),
        array($days, wc_price($reference_price)),
        $text_template
    );

    return '<span class="fleekcode-omnibus-price">' . wp_kses_post($custom_text) . '</span>';
}

add_shortcode('fleekcode_omnibus_price', 'fleekcode_omnibus_price_shortcode');

/**
 * Deactivation cleanup
 */
function fleekcode_omnibus_deactivate() {
    wp_cache_flush_group('fleekcode_admin');
    wp_cache_flush_group('omnibus');
}

/**
 * Uninstall cleanup
 */
function fleekcode_omnibus_uninstall() {
    if (!current_user_can('activate_plugins')) {
        return;
    }

    global $wpdb;

    $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
    $table_name = esc_sql($table_name);
    // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching, WordPress.DB.DirectDatabaseQuery.SchemaChange, WordPress.DB.PreparedSQL.InterpolatedNotPrepared
    $wpdb->query("DROP TABLE IF EXISTS $table_name");

    $options = [
        'fleekcode_omnibus_text',
        'fleekcode_omnibus_display_location',
        'fleekcode_omnibus_days',
        'fleekcode_omnibus_price_position',
        'fleekcode_omnibus_min_price_display_mode',
        'fleekcode_omnibus_display_method'
    ];
    foreach ($options as $option) {
        delete_option($option);
    }

    wp_cache_flush_group('fleekcode_admin');
    wp_cache_flush_group('omnibus');
}

/**
 * Notice for missing WooCommerce
 */
function fleekcode_omnibus_missing_wc_notice() {
    $message = sprintf(
    /* translators: %s: link to plugins page */
        esc_html__('FleekCode - Omnibus Price Tracker requires WooCommerce to be installed and active. %s', 'fleekcode-omnibus'),
        '<a href="' . esc_url(admin_url('plugins.php')) . '">' . esc_html__('Return to plugins page', 'fleekcode-omnibus') . '</a>'
    );
    printf('<div class="error"><p>%s</p></div>', wp_kses_post($message));
}

/**
 * Notice for incompatible WooCommerce version
 */
function fleekcode_omnibus_wc_version_notice() {
    $message = sprintf(
    /* translators: %s: required WooCommerce version */
        esc_html__('FleekCode - Omnibus Price Tracker requires WooCommerce version 3.0 or higher. Current version: %s', 'fleekcode-omnibus'),
        esc_html(WC()->version)
    );
    printf('<div class="error"><p>%s</p></div>', wp_kses_post($message));
}