<?php
class Fleekcode_Database {

    public static function init() {
        add_action('woocommerce_update_product', array(__CLASS__, 'save_product_price'));
        add_action('wp_ajax_fleekcode_omnibus_bulk_action', array(__CLASS__, 'bulk_action'));
    }

  
    public static function create_table() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE `$table_name` (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            product_id bigint(20) NOT NULL,
            regular_price decimal(15,2) NOT NULL,
            sale_price decimal(15,2) NOT NULL DEFAULT 0.00,
            date datetime NOT NULL,
            is_active tinyint(1) NOT NULL DEFAULT 1,
            PRIMARY KEY (id),
            KEY product_id (product_id)
        ) $charset_collate;";

        require_once ABSPATH . 'wp-admin/includes/upgrade.php';
        dbDelta($sql);
    }

    /**
     *
     * @param int $product_id.
     * @return void
     */
    public static function save_product_price($product_id) {
        if (!current_user_can('edit_post', $product_id)) {
            return;
        }

        $product = wc_get_product($product_id);
        if (!$product || !$product->is_type(['simple', 'variable'])) return;

        $new_regular = (float)$product->get_regular_price();
        $new_sale = (float)$product->get_sale_price();

        global $wpdb;
        $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
        $table_name = esc_sql($table_name);

        $transient_key = 'fleekcode_price_save_' . $product_id;
        if (get_transient($transient_key)) {
            return; 
        }
        set_transient($transient_key, true, 1);

        $last = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT regular_price, sale_price FROM `$table_name` WHERE product_id = %d AND is_active = 1 ORDER BY date DESC LIMIT 1",
                $product_id
            ),
            ARRAY_A
        );

        if ($last && (float)$last['regular_price'] === $new_regular && (float)$last['sale_price'] === $new_sale) {
            return; 
        }

        $wpdb->insert(
            $table_name,
            [
                'product_id' => $product_id,
                'regular_price' => $new_regular,
                'sale_price' => $new_sale,
                'date' => current_time('mysql'),
                'is_active' => 1
            ],
            ['%d', '%f', '%f', '%s', '%d']
        );

        wp_cache_delete('fleekcode_last_price_' . $product_id);
        wp_cache_delete('omnibus_price_' . $product_id, 'omnibus');
        wp_cache_delete('fleekcode_records_' . $product_id);
    }

    /**
     *
     * @param int $product_id
     * @return array 
     */
    public static function get_price_history($product_id) {
        $cache_key = 'fleekcode_price_history_' . $product_id;
        $history = wp_cache_get($cache_key, 'fleekcode_admin');

        if ($history === false) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
            $table_name = esc_sql($table_name);
            $history = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT * FROM `$table_name` WHERE product_id = %d ORDER BY date DESC",
                    $product_id
                ),
                ARRAY_A
            );
            wp_cache_set($cache_key, $history, 'fleekcode_admin');
        }

        return $history;
    }

    /**
     *
     * @param WC_Product $product
     * @return float|false
     */
    public static function get_reference_price($product) {
        $days = (int)get_option('fleekcode_omnibus_days', 30);
        $date_from = gmdate('Y-m-d H:i:s', strtotime("-$days days"));

        global $wpdb;
        $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
        $table_name = esc_sql($table_name);

        $product_id = $product->get_id();
        $cache_key = 'omnibus_price_' . $product_id;
        $reference_price = wp_cache_get($cache_key, 'omnibus');

        if ($reference_price === false) {
            $prices = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT sale_price, regular_price FROM `$table_name`
                    WHERE product_id = %d AND date >= %s AND is_active = 1
                    ORDER BY sale_price ASC, regular_price ASC",
                    $product_id,
                    $date_from
                ),
                ARRAY_A
            );

            if (empty($prices)) {
                return false;
            }

            $reference_price = null;
            $current_sale = (float)$product->get_sale_price();

            foreach ($prices as $price) {
                $price_to_compare = $price['sale_price'] > 0 ? $price['sale_price'] : $price['regular_price'];
                if ($reference_price === null || $price_to_compare < $reference_price) {
                    if ($current_sale && $price_to_compare != $current_sale) {
                        $reference_price = $price_to_compare;
                    } elseif (!$current_sale) {
                        $reference_price = $price_to_compare;
                    }
                }
            }

            if ($reference_price === null) {
                return false;
            }

            wp_cache_set($cache_key, $reference_price, 'omnibus');
        }

        return $reference_price;
    }

  
    public static function bulk_action() {
        check_ajax_referer('omnibus_bulk_action', 'nonce');

        if (!current_user_can('edit_posts')) {
            wp_send_json_error('Insufficient permissions');
        }

        $product_id = intval($_POST['product_id']);
        $record_ids = array_map('intval', $_POST['record_ids']);
        $action = sanitize_text_field($_POST['bulk_action']);

        global $wpdb;
        $table_name = $wpdb->prefix . 'fleekcode_omnibus_prices';
        $table_name = esc_sql($table_name);

        if ($action === 'activate') {
            $wpdb->query(
                $wpdb->prepare(
                    "UPDATE `$table_name` SET is_active = 1 WHERE product_id = %d AND id IN (" . implode(',', $record_ids) . ")",
                    $product_id
                )
            );
        } elseif ($action === 'deactivate') {
            $wpdb->query(
                $wpdb->prepare(
                    "UPDATE `$table_name` SET is_active = 0 WHERE product_id = %d AND id IN (" . implode(',', $record_ids) . ")",
                    $product_id
                )
            );
        }

        wp_cache_delete('fleekcode_records_' . $product_id);
        wp_cache_delete('omnibus_price_' . $product_id, 'omnibus');
        wp_cache_delete('fleekcode_price_history_' . $product_id, 'fleekcode_admin');

        wp_send_json_success();
    }
}